/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickHAT;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the HAT Brick.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickHATWrapper extends BrickHAT implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickHAT.DEVICE_DISPLAY_NAME, "brickhat",
            BrickHAT.DEVICE_IDENTIFIER, BrickHATWrapper.class, BrickHATActions.class, "2.0.1", true);

    public BrickHATWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickHATWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickHATConfig cfg = (BrickHATConfig) config.as(BrickHATConfig.class);
        this.addVoltagesListener(this.reg((voltageUSB, voltageDC) -> {
            if (true) {
                updateStateFn.accept("BrickHATUSBVoltage",
                        transformBrickHATUSBVoltageCallback0(voltageUSB, voltageDC, cfg));
            }
        }, this::removeVoltagesListener));
        this.addVoltagesListener(this.reg((voltageUSB, voltageDC) -> {
            if (true) {
                updateStateFn.accept("BrickHATDCVoltage",
                        transformBrickHATDCVoltageCallback0(voltageUSB, voltageDC, cfg));
            }
        }, this::removeVoltagesListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            this.setVoltagesCallbackConfiguration(cfg.voltagesUpdateInterval, true);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setVoltagesCallbackConfiguration(0, true);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickHATSleep");
        result.add("BrickHATPowerBricklets");
        result.add("BrickHATUSBVoltage");
        result.add("BrickHATDCVoltage");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickHATUSBVoltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickHATUSBVoltage"), "USB Voltage",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickHATUSBVoltage"))
                        .withDescription(
                                "The USB supply voltage.<br/><br/>There are three possible combinations:<ul><li>Only USB connected: The USB supply voltage will be fed back to the DC input connector. You will read the USB voltage and a slightly lower voltage on the DC input.</li><li>Only DC input connected: The DC voltage will not be fed back to the USB connector. You will read the DC input voltage and the USB voltage will be 0.</li><li>USB and DC input connected: You will read both voltages. In this case the USB supply will be without load, but it will work as backup if you disconnect the DC input (or if the DC input voltage falls below the USB voltage).</li></ul>")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickHATDCVoltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickHATDCVoltage"), "DC Voltage",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickHATDCVoltage"))
                        .withDescription(
                                "The DC supply voltage.<br/><br/>There are three possible combinations:<ul><li>Only USB connected: The USB supply voltage will be fed back to the DC input connector. You will read the USB voltage and a slightly lower voltage on the DC input.</li><li>Only DC input connected: The DC voltage will not be fed back to the USB connector. You will read the DC input voltage and the USB voltage will be 0.</li><li>USB and DC input connected: You will read both voltages. In this case the USB supply will be without load, but it will work as backup if you disconnect the DC input (or if the DC input voltage falls below the USB voltage).</li></ul>")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickHATPowerBricklets":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickHATPowerBricklets"), "Power Bricklets", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickHATPowerBricklets"))
                        .withDescription("Enable/disable to turn the power supply of the connected Bricklets on/off.")
                        .build();
            case "BrickHATSleep":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickHATSleep"), "Sleep", "String")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickHATSleep"))
                        .withDescription(
                                "Starts the configured sleep mode. Note: Triggering this will cut the Raspberry Pi's power after the configured amount of seconds. You have to shut down the operating system yourself, e.g. by calling 'sudo shutdown -h now'.")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("Trigger", "Start Sleep Mode")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge HAT Brick").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("HAT for Raspberry Pi with 8 Bricklets ports and real-time clock")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickHATSleep",
                                new ChannelTypeUID("tinkerforge", "BrickHATSleep")).withLabel("Sleep").build(),
                        new ChannelDefinitionBuilder("BrickHATPowerBricklets",
                                new ChannelTypeUID("tinkerforge", "BrickHATPowerBricklets"))
                                        .withLabel("Power Bricklets").build(),
                        new ChannelDefinitionBuilder("BrickHATUSBVoltage",
                                new ChannelTypeUID("tinkerforge", "BrickHATUSBVoltage")).withLabel("USB Voltage")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickHATDCVoltage",
                                new ChannelTypeUID("tinkerforge", "BrickHATDCVoltage")).withLabel("DC Voltage")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickhat":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("voltagesUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for the voltages in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Voltages Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(
                                new ConfigDescriptionParameterGroup("update_intervals", "null", true,
                                        "Update Intervals", "Update Intervals"),
                                new ConfigDescriptionParameterGroup("sleep", "null", true, "Sleep Mode",
                                        "Configures the sleep mode.")))
                        .build();
            case "channel-type:tinkerforge:BrickHATUSBVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickHATDCVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickHATPowerBricklets":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickHATSleep":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(
                                Arrays.asList(
                                        ConfigDescriptionParameterBuilder.create("powerOffDelay", Type.INTEGER)
                                                .withDefault("60")
                                                .withDescription(
                                                        "Time in seconds before the RPi/Bricklets are powered off.")
                                                .withGroupName("sleep").withLabel("Power Off Delay")
                                                .withMinimum(BigDecimal.valueOf(0))
                                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                                        ConfigDescriptionParameterBuilder.create("powerOffDuration", Type.INTEGER)
                                                .withDefault("10")
                                                .withDescription(
                                                        "Duration in seconds that the RPi/Bricklets stay powered off.")
                                                .withGroupName("sleep").withLabel("Power Off Duration")
                                                .withMinimum(BigDecimal.valueOf(0))
                                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                                        ConfigDescriptionParameterBuilder.create("raspberryPiOff", Type.BOOLEAN)
                                                .withDefault("false")
                                                .withDescription("Raspberry Pi is powered off if enabled.")
                                                .withGroupName("sleep").withLabel("Raspberry Pi Off").build(),
                                        ConfigDescriptionParameterBuilder.create("brickletsOff", Type.BOOLEAN)
                                                .withDefault("true")
                                                .withDescription("Bricklets are powered off if enabled.")
                                                .withGroupName("sleep").withLabel("Bricklets Off").build(),
                                        ConfigDescriptionParameterBuilder.create("sleepIndicator", Type.BOOLEAN)
                                                .withDefault("true")
                                                .withDescription(
                                                        "If enabled, the status LED will blink in a 1s interval during the whole power off duration. This will draw additional 0.3mA.")
                                                .withGroupName("sleep").withLabel("Sleep Indicator").build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickHATConfig cfg = (BrickHATConfig) config.as(BrickHATConfig.class);
        switch (channel) {
            case "BrickHATSleep":
                return;
            case "BrickHATPowerBricklets": {
                boolean value = this.getBrickletPower();
                if (true)
                    updateStateFn.accept(channel, transformBrickHATPowerBrickletsGetter0(value, cfg));
            }
                return;
            case "BrickHATUSBVoltage": {
                BrickHAT.Voltages value = this.getVoltages();
                if (true)
                    updateStateFn.accept(channel, transformBrickHATUSBVoltageGetter0(value, cfg));
            }
                return;
            case "BrickHATDCVoltage": {
                BrickHAT.Voltages value = this.getVoltages();
                if (true)
                    updateStateFn.accept(channel, transformBrickHATDCVoltageGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickHATSleep":
                if (command instanceof StringType) {
                    BrickHATSleepConfig channelCfg = channelConfig.as(BrickHATSleepConfig.class);
                    this.setSleepMode(channelCfg.powerOffDelay, channelCfg.powerOffDuration, channelCfg.raspberryPiOff,
                            channelCfg.brickletsOff, channelCfg.sleepIndicator);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickHATPowerBricklets":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setBrickletPower(cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickHATUSBVoltageCallback0(int voltageUSB, int voltageDC,
            BrickHATConfig cfg) {
        return new QuantityType<>(voltageUSB / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickHATDCVoltageCallback0(int voltageUSB, int voltageDC,
            BrickHATConfig cfg) {
        return new QuantityType<>(voltageDC / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickHATPowerBrickletsGetter0(boolean value,
            BrickHATConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickHATUSBVoltageGetter0(BrickHAT.Voltages value,
            BrickHATConfig cfg) {
        return new QuantityType<>(value.voltageUSB / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickHATDCVoltageGetter0(BrickHAT.Voltages value,
            BrickHATConfig cfg) {
        return new QuantityType<>(value.voltageDC / 1000.0, SmartHomeUnits.VOLT);
    }
}
