/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletAccelerometerV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Accelerometer Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletAccelerometerV2Wrapper extends BrickletAccelerometerV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletAccelerometerV2.DEVICE_DISPLAY_NAME,
            "brickletaccelerometerv2", BrickletAccelerometerV2.DEVICE_IDENTIFIER, BrickletAccelerometerV2Wrapper.class,
            BrickletAccelerometerV2Actions.class, "2.0.2", true);

    public BrickletAccelerometerV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletAccelerometerV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletAccelerometerV2Config cfg = (BrickletAccelerometerV2Config) config
                .as(BrickletAccelerometerV2Config.class);
        this.addAccelerationListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletAccelerometerV2AccelerationX",
                        transformBrickletAccelerometerV2AccelerationXCallback0(x, y, z, cfg));
            }
        }, this::removeAccelerationListener));
        this.addAccelerationListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletAccelerometerV2AccelerationY",
                        transformBrickletAccelerometerV2AccelerationYCallback0(x, y, z, cfg));
            }
        }, this::removeAccelerationListener));
        this.addAccelerationListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletAccelerometerV2AccelerationZ",
                        transformBrickletAccelerometerV2AccelerationZCallback0(x, y, z, cfg));
            }
        }, this::removeAccelerationListener));
        this.setConfiguration(cfg.dataRate, cfg.fullScaleRange);
        this.setFilterConfiguration(cfg.iirFilter ? 0 : 1, cfg.lowPassFilter);
        this.setInfoLEDConfig(cfg.infoLEDMode);
        this.setAccelerationCallbackConfiguration(cfg.accelerationUpdateInterval, true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletAccelerometerV2AccelerationX");
        result.add("BrickletAccelerometerV2AccelerationY");
        result.add("BrickletAccelerometerV2AccelerationZ");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletAccelerometerV2Acceleration":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletAccelerometerV2Acceleration"), "NOT USED",
                                "Number:Acceleration")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletAccelerometerV2Acceleration"))
                        .withDescription("The acceleration in g (1g = 9.80665m/s²), not to be confused with grams.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.4f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Accelerometer Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription(
                        "Measures acceleration in three axis")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletAccelerometerV2AccelerationX",
                                new ChannelTypeUID("tinkerforge", "BrickletAccelerometerV2Acceleration"))
                                        .withLabel("Acceleration X").build(),
                        new ChannelDefinitionBuilder("BrickletAccelerometerV2AccelerationY",
                                new ChannelTypeUID("tinkerforge", "BrickletAccelerometerV2Acceleration"))
                                        .withLabel("Acceleration Y").build(),
                        new ChannelDefinitionBuilder("BrickletAccelerometerV2AccelerationZ",
                                new ChannelTypeUID("tinkerforge", "BrickletAccelerometerV2Acceleration"))
                                        .withLabel("Acceleration Z").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletaccelerometerv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("dataRate", Type.INTEGER).withDefault("7")
                                .withDescription(
                                        "The data rate of 0.781Hz to 25600Hz. Decreasing data rate or full scale range will also decrease the noise on the data.")
                                .withLabel("Data Rate").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "0.781Hz"),
                                        new ParameterOption("1", "1.563Hz"), new ParameterOption("2", "3.125Hz"),
                                        new ParameterOption("3", "6.2512Hz"), new ParameterOption("4", "12.5Hz"),
                                        new ParameterOption("5", "25Hz"), new ParameterOption("6", "50Hz"),
                                        new ParameterOption("7", "100Hz"), new ParameterOption("8", "200Hz"),
                                        new ParameterOption("9", "400Hz"), new ParameterOption("10", "800Hz"),
                                        new ParameterOption("11", "1600Hz"), new ParameterOption("12", "3200Hz"),
                                        new ParameterOption("13", "6400Hz"), new ParameterOption("14", "12800Hz"),
                                        new ParameterOption("15", "25600Hz")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("fullScaleRange", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "Full scale range of ±2g up to ±8g. Decreasing data rate or full scale range will also decrease the noise on the data.")
                                .withLabel("Full Scale Range").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "2g"),
                                        new ParameterOption("1", "4g"), new ParameterOption("2", "8g")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("infoLEDMode", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "Configures the info LED (marked as \"Force\" on the Bricklet) to be either turned off, turned on, or blink in heartbeat mode.")
                                .withLabel("Info LED Mode").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("iirFilter", Type.BOOLEAN).withDefault("true")
                                .withDescription("Enables/Disables the IIR filter.").withLabel("IIR Filter")
                                .withAdvanced(true).build(),
                        ConfigDescriptionParameterBuilder.create("lowPassFilter", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "The low pass filter roll off corner frequency can be half or a ninth of the output data rate.")
                                .withLabel("Low Pass Filter Corner Frequency").withAdvanced(true)
                                .withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Ninth"),
                                        new ParameterOption("1", "Half")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("accelerationUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for the acceleration in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Acceleration Update Interval")
                                .withUnit("ms").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletAccelerometerV2Acceleration":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletAccelerometerV2Config cfg = (BrickletAccelerometerV2Config) config
                .as(BrickletAccelerometerV2Config.class);
        switch (channel) {
            case "BrickletAccelerometerV2AccelerationX": {
                BrickletAccelerometerV2.Acceleration value = this.getAcceleration();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAccelerometerV2AccelerationXGetter0(value, cfg));
            }
                return;
            case "BrickletAccelerometerV2AccelerationY": {
                BrickletAccelerometerV2.Acceleration value = this.getAcceleration();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAccelerometerV2AccelerationYGetter0(value, cfg));
            }
                return;
            case "BrickletAccelerometerV2AccelerationZ": {
                BrickletAccelerometerV2.Acceleration value = this.getAcceleration();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAccelerometerV2AccelerationZGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationXCallback0(int x, int y,
            int z, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(x / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationYCallback0(int x, int y,
            int z, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(y / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationZCallback0(int x, int y,
            int z, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(z / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationXGetter0(
            BrickletAccelerometerV2.Acceleration value, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(value.x / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationYGetter0(
            BrickletAccelerometerV2.Acceleration value, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(value.y / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerV2AccelerationZGetter0(
            BrickletAccelerometerV2.Acceleration value, BrickletAccelerometerV2Config cfg) {
        return new QuantityType<>(value.z / 10000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
}
