/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAmbientLight;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Ambient Light Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAmbientLightActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type = "int") Map<String, Object> brickletAmbientLightGetIlluminance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightWrapper dev = ((@Nullable BrickletAmbientLightWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getIlluminance();

        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightGetIlluminance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightActions) {
            return ((BrickletAmbientLightActions) actions).brickletAmbientLightGetIlluminance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type = "int") Map<String, Object> brickletAmbientLightGetAnalogValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightWrapper dev = ((@Nullable BrickletAmbientLightWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAnalogValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightGetAnalogValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightActions) {
            return ((BrickletAmbientLightActions) actions).brickletAmbientLightGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAmbientLightGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightWrapper dev = ((@Nullable BrickletAmbientLightWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLight.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightActions) {
            return ((BrickletAmbientLightActions) actions).brickletAmbientLightGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightActions class.");
        }
    }
}
