/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAnalogInV3;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Analog In Bricklet 3.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogInV3Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletAnalogInV3GetVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Oversampling")
    public @ActionOutput(name = "oversampling", type = "int") Map<String, Object> brickletAnalogInV3GetOversampling()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getOversampling();

        result.put("oversampling", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetOversampling(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetOversampling();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type = "int") @ActionOutput(name = "multiplier", type = "int") @ActionOutput(name = "divisor", type = "int") Map<String, Object> brickletAnalogInV3GetCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogInV3.Calibration value = dev.getCalibration();

        result.put("offset", value.offset);
        result.put("multiplier", value.multiplier);
        result.put("divisor", value.divisor);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletAnalogInV3ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletAnalogInV3GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletAnalogInV3GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletAnalogInV3GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogInV3.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAnalogInV3GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogInV3Wrapper dev = ((@Nullable BrickletAnalogInV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogInV3.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }
}
