/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAnalogOutV3;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Analog Out Bricklet 3.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutV3Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Output Voltage")
    public void brickletAnalogOutV3SetOutputVoltage(@ActionInput(name = "voltage") int voltage)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setOutputVoltage(voltage);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletAnalogOutV3OutputVoltage")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletAnalogOutV3OutputVoltage"), RefreshType.REFRESH);
        }
    }

    public static void brickletAnalogOutV3SetOutputVoltage(@Nullable ThingActions actions, int voltage)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3SetOutputVoltage(voltage);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletAnalogOutV3GetOutputVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getOutputVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetOutputVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetOutputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Input Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletAnalogOutV3GetInputVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getInputVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetInputVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletAnalogOutV3ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletAnalogOutV3GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletAnalogOutV3GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletAnalogOutV3GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogOutV3.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAnalogOutV3GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutV3Wrapper dev = ((@Nullable BrickletAnalogOutV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogOutV3.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }
}
