/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletBarometer;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Barometer Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletBarometerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type = "int") Map<String, Object> brickletBarometerGetAirPressure()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerWrapper dev = ((@Nullable BrickletBarometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAirPressure();

        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAirPressure(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type = "int") Map<String, Object> brickletBarometerGetAltitude()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerWrapper dev = ((@Nullable BrickletBarometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAltitude();

        result.put("altitude", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAltitude(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Reference Air Pressure")
    public @ActionOutput(name = "airPressure", type = "int") Map<String, Object> brickletBarometerGetReferenceAirPressure()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerWrapper dev = ((@Nullable BrickletBarometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getReferenceAirPressure();

        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetReferenceAirPressure(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetReferenceAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Averaging")
    public @ActionOutput(name = "movingAveragePressure", type = "short") @ActionOutput(name = "averagePressure", type = "short") @ActionOutput(name = "averageTemperature", type = "short") Map<String, Object> brickletBarometerGetAveraging()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerWrapper dev = ((@Nullable BrickletBarometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletBarometer.Averaging value = dev.getAveraging();

        result.put("movingAveragePressure", value.movingAveragePressure);
        result.put("averagePressure", value.averagePressure);
        result.put("averageTemperature", value.averageTemperature);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetAveraging(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetAveraging();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletBarometerGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerWrapper dev = ((@Nullable BrickletBarometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletBarometer.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletBarometerGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerActions) {
            return ((BrickletBarometerActions) actions).brickletBarometerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerActions class.");
        }
    }
}
