/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletBarometerV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Barometer Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletBarometerV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type = "int") Map<String, Object> brickletBarometerV2GetAirPressure()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAirPressure();

        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetAirPressure(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type = "int") Map<String, Object> brickletBarometerV2GetAltitude()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAltitude();

        result.put("altitude", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetAltitude(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletBarometerV2GetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthAirPressure", type = "int") @ActionOutput(name = "movingAverageLengthTemperature", type = "int") Map<String, Object> brickletBarometerV2GetMovingAverageConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletBarometerV2.MovingAverageConfiguration value = dev.getMovingAverageConfiguration();

        result.put("movingAverageLengthAirPressure", value.movingAverageLengthAirPressure);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetMovingAverageConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Reference Air Pressure")
    public @ActionOutput(name = "airPressure", type = "int") Map<String, Object> brickletBarometerV2GetReferenceAirPressure()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getReferenceAirPressure();

        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetReferenceAirPressure(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetReferenceAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletBarometerV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletBarometerV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletBarometerV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletBarometerV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletBarometerV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletBarometerV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletBarometerV2Wrapper dev = ((@Nullable BrickletBarometerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletBarometerV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }
}
