/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletCompass;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Compass Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCompassActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Heading")
    public @ActionOutput(name = "heading", type = "int") Map<String, Object> brickletCompassGetHeading()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getHeading();

        result.put("heading", value);
        return result;
    }

    public static Map<String, Object> brickletCompassGetHeading(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetHeading();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Magnetic Flux Density")
    public @ActionOutput(name = "x", type = "int") @ActionOutput(name = "y", type = "int") @ActionOutput(name = "z", type = "int") Map<String, Object> brickletCompassGetMagneticFluxDensity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletCompass.MagneticFluxDensity value = dev.getMagneticFluxDensity();

        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletCompassGetMagneticFluxDensity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetMagneticFluxDensity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "dataRate", type = "int") @ActionOutput(name = "backgroundCalibration", type = "boolean") Map<String, Object> brickletCompassGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletCompass.Configuration value = dev.getConfiguration();

        result.put("dataRate", value.dataRate);
        result.put("backgroundCalibration", value.backgroundCalibration);
        return result;
    }

    public static Map<String, Object> brickletCompassGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type = "int[]") @ActionOutput(name = "gain", type = "int[]") Map<String, Object> brickletCompassGetCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletCompass.Calibration value = dev.getCalibration();

        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletCompassGetCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletCompassReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletCompassReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletCompassGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletCompassGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletCompassGetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletCompassGetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletCompassGetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletCompass.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletCompassGetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletCompassGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletCompassWrapper dev = ((@Nullable BrickletCompassWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletCompass.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCompassGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).brickletCompassGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }
}
