/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialDigitalOut4V2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Digital Out 4 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDigitalOut4V2Wrapper extends BrickletIndustrialDigitalOut4V2
        implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDigitalOut4V2.DEVICE_DISPLAY_NAME,
            "brickletindustrialdigitalout4v2", BrickletIndustrialDigitalOut4V2.DEVICE_IDENTIFIER,
            BrickletIndustrialDigitalOut4V2Wrapper.class, BrickletIndustrialDigitalOut4V2Actions.class, "2.0.0", true);

    public BrickletIndustrialDigitalOut4V2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDigitalOut4V2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDigitalOut4V2Config cfg = (BrickletIndustrialDigitalOut4V2Config) config
                .as(BrickletIndustrialDigitalOut4V2Config.class);
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 0) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4V2OutputPin0",
                        transformBrickletIndustrialDigitalOut4V2OutputPin0Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 1) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4V2OutputPin1",
                        transformBrickletIndustrialDigitalOut4V2OutputPin1Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 2) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4V2OutputPin2",
                        transformBrickletIndustrialDigitalOut4V2OutputPin2Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 3) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4V2OutputPin3",
                        transformBrickletIndustrialDigitalOut4V2OutputPin3Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletIndustrialDigitalOut4V2OutputPinConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDigitalOut4V2OutputPin0")
                    .as(BrickletIndustrialDigitalOut4V2OutputPinConfig.class);
            this.setChannelLEDConfig(0, channelCfg.ledConfig);
        }
        {
            BrickletIndustrialDigitalOut4V2OutputPinConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDigitalOut4V2OutputPin1")
                    .as(BrickletIndustrialDigitalOut4V2OutputPinConfig.class);
            this.setChannelLEDConfig(1, channelCfg.ledConfig);
        }
        {
            BrickletIndustrialDigitalOut4V2OutputPinConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDigitalOut4V2OutputPin2")
                    .as(BrickletIndustrialDigitalOut4V2OutputPinConfig.class);
            this.setChannelLEDConfig(2, channelCfg.ledConfig);
        }
        {
            BrickletIndustrialDigitalOut4V2OutputPinConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDigitalOut4V2OutputPin3")
                    .as(BrickletIndustrialDigitalOut4V2OutputPinConfig.class);
            this.setChannelLEDConfig(3, channelCfg.ledConfig);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDigitalOut4V2OutputPin0");
        result.add("BrickletIndustrialDigitalOut4V2OutputPin1");
        result.add("BrickletIndustrialDigitalOut4V2OutputPin2");
        result.add("BrickletIndustrialDigitalOut4V2OutputPin3");
        result.add("BrickletIndustrialDigitalOut4V2MonoflopPin0");
        result.add("BrickletIndustrialDigitalOut4V2MonoflopPin1");
        result.add("BrickletIndustrialDigitalOut4V2MonoflopPin2");
        result.add("BrickletIndustrialDigitalOut4V2MonoflopPin3");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialDigitalOut4V2OutputPin":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2OutputPin"),
                                "Output Value", "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalOut4V2OutputPin"))
                        .withDescription("The logic level that is currently set on the pin.").build();
            case "BrickletIndustrialDigitalOut4V2Monoflop":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2Monoflop"), "NOT USED",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalOut4V2Monoflop"))
                        .withDescription("Triggers a monoflop as configured")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TRIGGER", "Trigger")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Digital Out 4 Bricklet 2.0")
                .isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("4 galvanically isolated digital outputs")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2OutputPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2OutputPin"))
                                        .withLabel("Output Value (Pin 0)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2OutputPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2OutputPin"))
                                        .withLabel("Output Value (Pin 1)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2OutputPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2OutputPin"))
                                        .withLabel("Output Value (Pin 2)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2OutputPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2OutputPin"))
                                        .withLabel("Output Value (Pin 3)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2MonoflopPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2Monoflop"))
                                        .withLabel("Monoflop Pin 0").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2MonoflopPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2Monoflop"))
                                        .withLabel("Monoflop Pin 1").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2MonoflopPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2Monoflop"))
                                        .withLabel("Monoflop Pin 2").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4V2MonoflopPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4V2Monoflop"))
                                        .withLabel("Monoflop Pin 3").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdigitalout4v2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalOut4V2OutputPin":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("ledConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Each channel has a corresponding LED. You can turn the LED off, on or show a heartbeat. You can also set the LED to \"Channel Status\". In this mode the LED is on if the channel is high and off otherwise.")
                                .withLabel("LED Configuration").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Channel Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalOut4V2Monoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000")
                                .withDescription("The time (in ms) that the pin should hold the configured value.")
                                .withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.")
                                .withLabel("Monoflop Value").withAdvanced(true).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDigitalOut4V2Config cfg = (BrickletIndustrialDigitalOut4V2Config) config
                .as(BrickletIndustrialDigitalOut4V2Config.class);
        switch (channel) {
            case "BrickletIndustrialDigitalOut4V2OutputPin0": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2OutputPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2OutputPin1": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2OutputPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2OutputPin2": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2OutputPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2OutputPin3": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2OutputPin3Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin0": {
                BrickletIndustrialDigitalOut4V2.Monoflop value = this.getMonoflop(0);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2MonoflopPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin1": {
                BrickletIndustrialDigitalOut4V2.Monoflop value = this.getMonoflop(1);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2MonoflopPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin2": {
                BrickletIndustrialDigitalOut4V2.Monoflop value = this.getMonoflop(2);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2MonoflopPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin3": {
                BrickletIndustrialDigitalOut4V2.Monoflop value = this.getMonoflop(3);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4V2MonoflopPin3Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletIndustrialDigitalOut4V2OutputPin0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(0, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4V2OutputPin1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(1, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4V2OutputPin2":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(2, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4V2OutputPin3":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(3, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin0":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4V2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4V2MonoflopConfig.class);
                    this.setMonoflop(0, channelCfg.monoflopValue.booleanValue(),
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4V2OutputPin0", 0));
                break;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin1":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4V2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4V2MonoflopConfig.class);
                    this.setMonoflop(1, channelCfg.monoflopValue.booleanValue(),
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4V2OutputPin1", 0));
                break;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin2":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4V2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4V2MonoflopConfig.class);
                    this.setMonoflop(2, channelCfg.monoflopValue.booleanValue(),
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4V2OutputPin2", 0));
                break;
            case "BrickletIndustrialDigitalOut4V2MonoflopPin3":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4V2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4V2MonoflopConfig.class);
                    this.setMonoflop(3, channelCfg.monoflopValue.booleanValue(),
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4V2OutputPin3", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin0Callback0(
            int channel, boolean value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin1Callback0(
            int channel, boolean value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin2Callback0(
            int channel, boolean value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin3Callback0(
            int channel, boolean value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin0Getter0(
            boolean[] value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value[0] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin1Getter0(
            boolean[] value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value[1] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin2Getter0(
            boolean[] value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value[2] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2OutputPin3Getter0(
            boolean[] value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value[3] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2MonoflopPin0Getter0(
            BrickletIndustrialDigitalOut4V2.Monoflop value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2MonoflopPin1Getter0(
            BrickletIndustrialDigitalOut4V2.Monoflop value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2MonoflopPin2Getter0(
            BrickletIndustrialDigitalOut4V2.Monoflop value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4V2MonoflopPin3Getter0(
            BrickletIndustrialDigitalOut4V2.Monoflop value, BrickletIndustrialDigitalOut4V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }
}
