/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletIndustrialQuadRelayV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Industrial Quad Relay Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialQuadRelayV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Value")
    public void brickletIndustrialQuadRelayV2SetValue(@ActionInput(name = "value") boolean[] value)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setValue(value);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3"),
                    RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialQuadRelayV2SetValue(@Nullable ThingActions actions, boolean[] value)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2SetValue(value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Set Selected Value")
    public void brickletIndustrialQuadRelayV2SetSelectedValue(@ActionInput(name = "channel") int channel,
            @ActionInput(name = "value") boolean value) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSelectedValue(channel, value);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3"),
                    RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialQuadRelayV2SetSelectedValue(@Nullable ThingActions actions, int channel,
            boolean value) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2SetSelectedValue(channel,
                    value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletIndustrialQuadRelayV2SetMonoflop(@ActionInput(name = "channel") int channel,
            @ActionInput(name = "value") boolean value, @ActionInput(name = "time") long time)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMonoflop(channel, value, time);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay0"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2Relay3"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay0"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay1"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay2"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialQuadRelayV2MonoflopRelay3"),
                    RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialQuadRelayV2SetMonoflop(@Nullable ThingActions actions, int channel,
            boolean value, long time) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2SetMonoflop(channel, value,
                    time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type = "boolean[]") Map<String, Object> brickletIndustrialQuadRelayV2GetValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean[] value = dev.getValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletIndustrialQuadRelayV2GetChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChannelLEDConfig(channel);

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetChannelLEDConfig(@Nullable ThingActions actions,
            int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions)
                    .brickletIndustrialQuadRelayV2GetChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type = "boolean") @ActionOutput(name = "time", type = "long") @ActionOutput(name = "timeRemaining", type = "long") Map<String, Object> brickletIndustrialQuadRelayV2GetMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialQuadRelayV2.Monoflop value = dev.getMonoflop(channel);

        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetMonoflop(@Nullable ThingActions actions,
            int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletIndustrialQuadRelayV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletIndustrialQuadRelayV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletIndustrialQuadRelayV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletIndustrialQuadRelayV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialQuadRelayV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletIndustrialQuadRelayV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialQuadRelayV2Wrapper dev = ((@Nullable BrickletIndustrialQuadRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialQuadRelayV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }
}
