/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialQuadRelayV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Quad Relay Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialQuadRelayV2Wrapper extends BrickletIndustrialQuadRelayV2
        implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialQuadRelayV2.DEVICE_DISPLAY_NAME,
            "brickletindustrialquadrelayv2", BrickletIndustrialQuadRelayV2.DEVICE_IDENTIFIER,
            BrickletIndustrialQuadRelayV2Wrapper.class, BrickletIndustrialQuadRelayV2Actions.class, "2.0.0", true);

    public BrickletIndustrialQuadRelayV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialQuadRelayV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialQuadRelayV2Config cfg = (BrickletIndustrialQuadRelayV2Config) config
                .as(BrickletIndustrialQuadRelayV2Config.class);
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 0) {
                updateStateFn.accept("BrickletIndustrialQuadRelayV2Relay0",
                        transformBrickletIndustrialQuadRelayV2Relay0Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 1) {
                updateStateFn.accept("BrickletIndustrialQuadRelayV2Relay1",
                        transformBrickletIndustrialQuadRelayV2Relay1Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 2) {
                updateStateFn.accept("BrickletIndustrialQuadRelayV2Relay2",
                        transformBrickletIndustrialQuadRelayV2Relay2Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 3) {
                updateStateFn.accept("BrickletIndustrialQuadRelayV2Relay3",
                        transformBrickletIndustrialQuadRelayV2Relay3Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletIndustrialQuadRelayV2RelayConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialQuadRelayV2Relay0").as(BrickletIndustrialQuadRelayV2RelayConfig.class);
            this.setChannelLEDConfig(0, channelCfg.channelLEDConfig);
        }
        {
            BrickletIndustrialQuadRelayV2RelayConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialQuadRelayV2Relay1").as(BrickletIndustrialQuadRelayV2RelayConfig.class);
            this.setChannelLEDConfig(1, channelCfg.channelLEDConfig);
        }
        {
            BrickletIndustrialQuadRelayV2RelayConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialQuadRelayV2Relay2").as(BrickletIndustrialQuadRelayV2RelayConfig.class);
            this.setChannelLEDConfig(2, channelCfg.channelLEDConfig);
        }
        {
            BrickletIndustrialQuadRelayV2RelayConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialQuadRelayV2Relay3").as(BrickletIndustrialQuadRelayV2RelayConfig.class);
            this.setChannelLEDConfig(3, channelCfg.channelLEDConfig);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialQuadRelayV2Relay0");
        result.add("BrickletIndustrialQuadRelayV2Relay1");
        result.add("BrickletIndustrialQuadRelayV2Relay2");
        result.add("BrickletIndustrialQuadRelayV2Relay3");
        result.add("BrickletIndustrialQuadRelayV2MonoflopRelay0");
        result.add("BrickletIndustrialQuadRelayV2MonoflopRelay1");
        result.add("BrickletIndustrialQuadRelayV2MonoflopRelay2");
        result.add("BrickletIndustrialQuadRelayV2MonoflopRelay3");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialQuadRelayV2Relay":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Relay"), "NOT USED",
                                "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialQuadRelayV2Relay"))
                        .withDescription("NOT USED").build();
            case "BrickletIndustrialQuadRelayV2Monoflop":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Monoflop"), "NOT USED",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialQuadRelayV2Monoflop"))
                        .withDescription("Triggers a monoflop as configured")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TRIGGER", "Trigger")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Quad Relay Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("4 galvanically isolated solid state relays")
                .withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder(
                        "BrickletIndustrialQuadRelayV2Relay0",
                        new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Relay"))
                                .withLabel("Relay 0")
                                .withDescription(
                                        "Switches Relay 0. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2Relay1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Relay"))
                                        .withLabel("Relay 1")
                                        .withDescription(
                                                "Switches Relay 1. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2Relay2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Relay"))
                                        .withLabel("Relay 2")
                                        .withDescription(
                                                "Switches Relay 2. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2Relay3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Relay"))
                                        .withLabel("Relay 3")
                                        .withDescription(
                                                "Switches Relay 3. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2MonoflopRelay0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Monoflop"))
                                        .withLabel("Monoflop Relay 0").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2MonoflopRelay1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Monoflop"))
                                        .withLabel("Monoflop Relay 1").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2MonoflopRelay2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Monoflop"))
                                        .withLabel("Monoflop Relay 2").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialQuadRelayV2MonoflopRelay3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialQuadRelayV2Monoflop"))
                                        .withLabel("Monoflop Relay 3").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialquadrelayv2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletIndustrialQuadRelayV2Relay":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("channelLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Each channel has a corresponding LED. You can turn the LED off, on or show a heartbeat. You can also set the LED to Channel Status. In this mode the LED is on if the channel is high and off otherwise.")
                                .withLabel("Channel LED Config").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Channel Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialQuadRelayV2Monoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000")
                                .withDescription("The time (in ms) that the relay should hold the configured value.")
                                .withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.")
                                .withLabel("Monoflop Value").withAdvanced(true).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialQuadRelayV2Config cfg = (BrickletIndustrialQuadRelayV2Config) config
                .as(BrickletIndustrialQuadRelayV2Config.class);
        switch (channel) {
            case "BrickletIndustrialQuadRelayV2Relay0": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialQuadRelayV2Relay0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2Relay1": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialQuadRelayV2Relay1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2Relay2": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialQuadRelayV2Relay2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2Relay3": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialQuadRelayV2Relay3Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay0": {
                BrickletIndustrialQuadRelayV2.Monoflop value = this.getMonoflop(0);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialQuadRelayV2MonoflopRelay0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay1": {
                BrickletIndustrialQuadRelayV2.Monoflop value = this.getMonoflop(1);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialQuadRelayV2MonoflopRelay1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay2": {
                BrickletIndustrialQuadRelayV2.Monoflop value = this.getMonoflop(2);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialQuadRelayV2MonoflopRelay2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay3": {
                BrickletIndustrialQuadRelayV2.Monoflop value = this.getMonoflop(3);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialQuadRelayV2MonoflopRelay3Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletIndustrialQuadRelayV2Relay0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(0, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialQuadRelayV2Relay1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(1, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialQuadRelayV2Relay2":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(2, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialQuadRelayV2Relay3":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(3, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay0":
                if (command instanceof StringType) {
                    BrickletIndustrialQuadRelayV2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialQuadRelayV2MonoflopConfig.class);
                    this.setMonoflop(0, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialQuadRelayV2Relay0", 0));
                break;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay1":
                if (command instanceof StringType) {
                    BrickletIndustrialQuadRelayV2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialQuadRelayV2MonoflopConfig.class);
                    this.setMonoflop(1, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialQuadRelayV2Relay1", 0));
                break;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay2":
                if (command instanceof StringType) {
                    BrickletIndustrialQuadRelayV2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialQuadRelayV2MonoflopConfig.class);
                    this.setMonoflop(2, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialQuadRelayV2Relay2", 0));
                break;
            case "BrickletIndustrialQuadRelayV2MonoflopRelay3":
                if (command instanceof StringType) {
                    BrickletIndustrialQuadRelayV2MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialQuadRelayV2MonoflopConfig.class);
                    this.setMonoflop(3, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialQuadRelayV2Relay3", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay0Callback0(int channel,
            boolean value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay1Callback0(int channel,
            boolean value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay2Callback0(int channel,
            boolean value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay3Callback0(int channel,
            boolean value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay0Getter0(boolean[] value,
            BrickletIndustrialQuadRelayV2Config cfg) {
        return value[0] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay1Getter0(boolean[] value,
            BrickletIndustrialQuadRelayV2Config cfg) {
        return value[1] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay2Getter0(boolean[] value,
            BrickletIndustrialQuadRelayV2Config cfg) {
        return value[2] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2Relay3Getter0(boolean[] value,
            BrickletIndustrialQuadRelayV2Config cfg) {
        return value[3] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2MonoflopRelay0Getter0(
            BrickletIndustrialQuadRelayV2.Monoflop value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2MonoflopRelay1Getter0(
            BrickletIndustrialQuadRelayV2.Monoflop value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2MonoflopRelay2Getter0(
            BrickletIndustrialQuadRelayV2.Monoflop value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialQuadRelayV2MonoflopRelay3Getter0(
            BrickletIndustrialQuadRelayV2.Monoflop value, BrickletIndustrialQuadRelayV2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }
}
