/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletJoystickV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Joystick Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletJoystickV2Wrapper extends BrickletJoystickV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletJoystickV2.DEVICE_DISPLAY_NAME,
            "brickletjoystickv2", BrickletJoystickV2.DEVICE_IDENTIFIER, BrickletJoystickV2Wrapper.class,
            BrickletJoystickV2Actions.class, "2.0.0", true);

    public BrickletJoystickV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletJoystickV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletJoystickV2Config cfg = (BrickletJoystickV2Config) config.as(BrickletJoystickV2Config.class);
        this.addPositionListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickV2PositionX",
                        transformBrickletJoystickV2PositionXCallback0(x, y, cfg));
            }
        }, this::removePositionListener));
        this.addPositionListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickV2PositionY",
                        transformBrickletJoystickV2PositionYCallback0(x, y, cfg));
            }
        }, this::removePositionListener));
        this.addPressedListener(this.reg((pressed) -> {
            if (true) {
                triggerChannelFn.accept("BrickletJoystickV2Pressed",
                        transformBrickletJoystickV2PressedCallback0(pressed, cfg));
            }
        }, this::removePressedListener));
        this.setPositionCallbackConfiguration(cfg.positionUpdateInterval, true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            this.setPressedCallbackConfiguration(10, true);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setPositionCallbackConfiguration(0, true);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletJoystickV2PositionX");
        result.add("BrickletJoystickV2PositionY");
        result.add("BrickletJoystickV2Pressed");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletJoystickV2Position":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletJoystickV2Position"), "Position",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletJoystickV2Position"))
                        .withDescription(
                                "The position of the joystick. The value ranges between -100 and 100 for both axis. The middle position of the joystick is x=0, y=0. The returned values are averaged and calibrated.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-100))
                                        .withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Joystick Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("2-axis joystick with push-button")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletJoystickV2PositionX",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickV2Position")).withLabel("Position X")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletJoystickV2PositionY",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickV2Position")).withLabel("Position Y")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletJoystickV2Pressed",
                                new ChannelTypeUID("system", "rawbutton")).withLabel("Pressed")
                                        .withDescription("Triggers if the button is pressed or released").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletjoystickv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("positionUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for X and Y position in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Position Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletJoystickV2Position":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletJoystickV2Config cfg = (BrickletJoystickV2Config) config.as(BrickletJoystickV2Config.class);
        switch (channel) {
            case "BrickletJoystickV2PositionX": {
                BrickletJoystickV2.Position value = this.getPosition();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickV2PositionXGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickV2PositionY": {
                BrickletJoystickV2.Position value = this.getPosition();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickV2PositionYGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickV2Pressed": {
                boolean value = this.isPressed();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletJoystickV2PressedGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickV2PositionXCallback0(int x, int y,
            BrickletJoystickV2Config cfg) {
        return new DecimalType(x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickV2PositionYCallback0(int x, int y,
            BrickletJoystickV2Config cfg) {
        return new DecimalType(y);
    }

    private String transformBrickletJoystickV2PressedCallback0(boolean pressed, BrickletJoystickV2Config cfg) {
        return pressed ? CommonTriggerEvents.PRESSED : CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickV2PositionXGetter0(
            BrickletJoystickV2.Position value, BrickletJoystickV2Config cfg) {
        return new DecimalType(value.x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickV2PositionYGetter0(
            BrickletJoystickV2.Position value, BrickletJoystickV2Config cfg) {
        return new DecimalType(value.y);
    }

    private String transformBrickletJoystickV2PressedGetter0(boolean value, BrickletJoystickV2Config cfg) {
        return value ? CommonTriggerEvents.PRESSED : CommonTriggerEvents.RELEASED;
    }
}
