/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.HSBType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletLEDStripV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the LED Strip Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletLEDStripV2Wrapper extends BrickletLEDStripV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletLEDStripV2.DEVICE_DISPLAY_NAME,
            "brickletledstripv2", BrickletLEDStripV2.DEVICE_IDENTIFIER, BrickletLEDStripV2Wrapper.class,
            BrickletLEDStripV2Actions.class, "2.0.0", true);

    public BrickletLEDStripV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletLEDStripV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletLEDStripV2Config cfg = (BrickletLEDStripV2Config) config.as(BrickletLEDStripV2Config.class);
        this.addFrameStartedListener(this.reg((length) -> {
            if (true) {
                triggerChannelFn.accept("BrickletLEDStripV2FrameStarted",
                        transformBrickletLEDStripV2FrameStartedCallback0(length, cfg));
            }
        }, this::removeFrameStartedListener));
        this.setChipType(cfg.chipType);
        this.setChannelMapping(cfg.channelMapping);
        this.setFrameDuration(cfg.frameDuration);
        this.setClockFrequency(cfg.clockFrequency.longValue());
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            this.setFrameStartedCallbackConfiguration(true);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setFrameStartedCallbackConfiguration(false);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletLEDStripV2FrameStarted");
        result.add("BrickletLEDStripV2LEDValues");
        result.add("BrickletLEDStripV2AllLEDs");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletLEDStripV2LEDValues":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLEDStripV2LEDValues"), "LED Values", "String")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLEDStripV2LEDValues"))
                        .withDescription(
                                "The RGB(W) values for the LEDs. Changes will be applied the next time the Frame Started Channel triggers.<br/><br/>Command format is a ','-separated list of integers. The first integer is the index of the first LED to set, additional integers are the values to set. Values are between 0 (off) and 255 (on). If the channel mapping has 3 colors, you need to give the data in the sequence R,G,B,R,G,B,R,G,B,... if the channel mapping has 4 colors you need to give data in the sequence R,G,B,W,R,G,B,W,R,G,B,W...<br/><br/>The data is double buffered and the colors will be transfered to the LEDs when the next frame duration ends. You can set at most 2048 RGB values or 1536 RGBW values.<br/><br/> For example sending 2,255,0,0,0,255,0,0,0,255 will set the LED 2 to red, LED 3 to green and LED 4 to blue.")
                        .build();
            case "BrickletLEDStripV2AllLEDs":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLEDStripV2AllLEDs"), "All LEDs", "Color")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLEDStripV2AllLEDs"))
                        .withDescription(
                                "This channel allows you to set a configurable amount of LEDs (up to 1536 RGBW LEDs or 2048 RGB LEDs) to the same color. If you want more fine-grained control over the LEDs, use the LED Values channel or the actions.")
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge LED Strip Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Controls up to 2048 RGB(W) LEDs")
                .withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletLEDStripV2FrameStarted",
                        new ChannelTypeUID("system", "trigger")).withLabel("Frame Started").withDescription(
                                "This channel is triggered directly after a new frame render is started. You should send the data for the next frame directly after this listener was triggered.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickletLEDStripV2LEDValues",
                                new ChannelTypeUID("tinkerforge", "BrickletLEDStripV2LEDValues"))
                                        .withLabel("LED Values").build(),
                        new ChannelDefinitionBuilder("BrickletLEDStripV2AllLEDs",
                                new ChannelTypeUID("tinkerforge", "BrickletLEDStripV2AllLEDs")).withLabel("All LEDs")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletledstripv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("chipType", Type.INTEGER).withDefault("2801")
                                .withDescription(
                                        "The type of the LED driver chip. We currently support the chips<br/><br/><ul><li>WS2801</li><li>WS2811</li><li>WS2812 / SK6812 / NeoPixel RGB</li><li>SK6812RGBW / NeoPixel RGBW (Chip Type = WS2812)</li><li>LPD8806</li><li>APA102 / DotStar</li></ul>")
                                .withLabel("LED Driver Chip Type").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("2801", "WS2801"),
                                        new ParameterOption("2811", "WS2811"), new ParameterOption("2812", "WS2812"),
                                        new ParameterOption("8806", "LPD8806"), new ParameterOption("102", "APA102")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("channelMapping", Type.INTEGER).withDefault("36")
                                .withDescription(
                                        "The channel mapping for the connected LEDs.<br/><br/>If the mapping has 4 colors, the LED Values channel expects 4 values per pixel and if the mapping has 3 colors it expects 3 values per pixel.<br/><br/>The LED Values channel always expects the order RGB(W). The connected LED driver chips might have their 3 or 4 channels in a different order. For example, the WS2801 chips typically use BGR order, then WS2812 chips typically use GRB order and the APA102 chips typically use WBGR order.<br/><br/>The APA102 chips are special. They have three 8-bit channels for RGB and an additional 5-bit channel for the overall brightness of the RGB LED making them 4-channel chips. Internally the brightness channel is the first channel, therefore one of the Wxyz channel mappings should be used. Then the W channel controls the brightness.")
                                .withLabel("LED Channel Mapping").withLimitToOptions(true)
                                .withOptions(
                                        Arrays.asList(new ParameterOption("6", "RGB"), new ParameterOption("9", "RBG"),
                                                new ParameterOption("33", "BRG"), new ParameterOption("36", "BGR"),
                                                new ParameterOption("18", "GRB"), new ParameterOption("24", "GBR"),
                                                new ParameterOption("27", "RGBW"), new ParameterOption("30", "RGWB"),
                                                new ParameterOption("39", "RBGW"), new ParameterOption("45", "RBWG"),
                                                new ParameterOption("54", "RWGB"), new ParameterOption("57", "RWBG"),
                                                new ParameterOption("78", "GRWB"), new ParameterOption("75", "GRBW"),
                                                new ParameterOption("108", "GBWR"), new ParameterOption("99", "GBRW"),
                                                new ParameterOption("120", "GWBR"), new ParameterOption("114", "GWRB"),
                                                new ParameterOption("135", "BRGW"), new ParameterOption("141", "BRWG"),
                                                new ParameterOption("147", "BGRW"), new ParameterOption("156", "BGWR"),
                                                new ParameterOption("177", "BWRG"), new ParameterOption("180", "BWGR"),
                                                new ParameterOption("201", "WRBG"), new ParameterOption("198", "WRGB"),
                                                new ParameterOption("216", "WGBR"), new ParameterOption("210", "WGRB"),
                                                new ParameterOption("228", "WBGR"), new ParameterOption("225", "WBRG")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("frameDuration", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "The frame duration in milliseconds. This configures how fast the Frame Started Channel will trigger.")
                                .withLabel("Frame Duration").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(65535)).build(),
                        ConfigDescriptionParameterBuilder.create("clockFrequency", Type.INTEGER).withDefault("1666666")
                                .withDescription(
                                        "The frequency of the clock in Hz. The Bricklet will choose the nearest achievable frequency, which may be off by a few Hz.<br/><br/>If you have problems with flickering LEDs, they may be bits flipping. You can fix this by either making the connection between the LEDs and the Bricklet shorter or by reducing the frequency.<br/><br/>With a decreasing frequency your maximum frames per second will decrease too.<br/><br/>The default value is 1.66MHz (1660000Hz).")
                                .withLabel("Clock Frequency").withMinimum(BigDecimal.valueOf(10000))
                                .withMaximum(BigDecimal.valueOf(2000000)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletLEDStripV2LEDValues":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLEDStripV2AllLEDs":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("ledCount", Type.INTEGER)
                                .withDefault("0").withDescription("The number of LEDs to control.")
                                .withLabel("LED Count").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(2048)).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        switch (channel) {
            case "BrickletLEDStripV2FrameStarted":
                return;
            case "BrickletLEDStripV2LEDValues":
                return;
            case "BrickletLEDStripV2AllLEDs":
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        BrickletLEDStripV2Config cfg = (BrickletLEDStripV2Config) config.as(BrickletLEDStripV2Config.class);
        switch (channel) {
            case "BrickletLEDStripV2LEDValues":
                if (command instanceof StringType) {
                    StringType cmd = (StringType) command;
                    this.setLEDValues(
                            Helper.parseLED2ValueIndex(cmd.toString(),
                                    !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping), logger),
                            Helper.parseLEDValues(cmd.toString(), logger));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletLEDStripV2AllLEDs":
                if (command instanceof HSBType) {
                    BrickletLEDStripV2AllLEDsConfig channelCfg = channelConfig
                            .as(BrickletLEDStripV2AllLEDsConfig.class);
                    HSBType cmd = (HSBType) command;
                    if (Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
                        this.setLEDValues(0,
                                Helper.createLED2ColorComponentList(cmd, false, Math.min(channelCfg.ledCount, 2048)));
                    }
                }

                if (command instanceof HSBType) {
                    BrickletLEDStripV2AllLEDsConfig channelCfg = channelConfig
                            .as(BrickletLEDStripV2AllLEDsConfig.class);
                    HSBType cmd = (HSBType) command;
                    if (!Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
                        this.setLEDValues(0,
                                Helper.createLED2ColorComponentList(cmd, true, Math.min(channelCfg.ledCount, 1536)));
                    }
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of HSBType, HSBType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private String transformBrickletLEDStripV2FrameStartedCallback0(int length, BrickletLEDStripV2Config cfg) {
        return "";
    }
}
