/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletLoadCell;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Load Cell Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLoadCellActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Weight")
    public @ActionOutput(name = "weight", type = "int") Map<String, Object> brickletLoadCellGetWeight()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getWeight();

        result.put("weight", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetWeight(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetWeight();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Tare")
    public void brickletLoadCellTare() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.tare();
    }

    public static void brickletLoadCellTare(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            ((BrickletLoadCellActions) actions).brickletLoadCellTare();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type = "short") Map<String, Object> brickletLoadCellGetMovingAverage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getMovingAverage();

        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetMovingAverage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "rate", type = "short") @ActionOutput(name = "gain", type = "short") Map<String, Object> brickletLoadCellGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLoadCell.Configuration value = dev.getConfiguration();

        result.put("rate", value.rate);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "LED On")
    public void brickletLoadCellLEDOn() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.ledOn();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletLoadCellLED")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletLoadCellLED"), RefreshType.REFRESH);
        }
    }

    public static void brickletLoadCellLEDOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            ((BrickletLoadCellActions) actions).brickletLoadCellLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "LED Off")
    public void brickletLoadCellLEDOff() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.ledOff();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletLoadCellLED")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletLoadCellLED"), RefreshType.REFRESH);
        }
    }

    public static void brickletLoadCellLEDOff(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            ((BrickletLoadCellActions) actions).brickletLoadCellLEDOff();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Is LED On")
    public @ActionOutput(name = "on", type = "boolean") Map<String, Object> brickletLoadCellIsLEDOn()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isLEDOn();

        result.put("on", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellIsLEDOn(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellIsLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletLoadCellGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellWrapper dev = ((@Nullable BrickletLoadCellWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLoadCell.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLoadCellGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellActions) {
            return ((BrickletLoadCellActions) actions).brickletLoadCellGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellActions class.");
        }
    }
}
