/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletMotionDetectorV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Motion Detector Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotionDetectorV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Motion Detected")
    public @ActionOutput(name = "motion", type = "int") Map<String, Object> brickletMotionDetectorV2GetMotionDetected()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getMotionDetected();

        result.put("motion", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetMotionDetected(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetMotionDetected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensitivity")
    public @ActionOutput(name = "sensitivity", type = "int") Map<String, Object> brickletMotionDetectorV2GetSensitivity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getSensitivity();

        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSensitivity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Set Indicator")
    public void brickletMotionDetectorV2SetIndicator(@ActionInput(name = "topLeft") int topLeft,
            @ActionInput(name = "topRight") int topRight, @ActionInput(name = "bottom") int bottom)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setIndicator(topLeft, topRight, bottom);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopLeftIndicator")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopLeftIndicator"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopRightIndicator")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopRightIndicator"),
                    RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2BottomIndicator")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2BottomIndicator"), RefreshType.REFRESH);
        }
    }

    public static void brickletMotionDetectorV2SetIndicator(@Nullable ThingActions actions, int topLeft, int topRight,
            int bottom) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2SetIndicator(topLeft, topRight, bottom);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Indicator")
    public @ActionOutput(name = "topLeft", type = "int") @ActionOutput(name = "topRight", type = "int") @ActionOutput(name = "bottom", type = "int") Map<String, Object> brickletMotionDetectorV2GetIndicator()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.Indicator value = dev.getIndicator();

        result.put("topLeft", value.topLeft);
        result.put("topRight", value.topRight);
        result.put("bottom", value.bottom);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIndicator(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetIndicator();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletMotionDetectorV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletMotionDetectorV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletMotionDetectorV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).brickletMotionDetectorV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }
}
