/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletOLED128x64;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the OLED 128x64 Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED128x64Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Write")
    public void brickletOLED128x64Write(@ActionInput(name = "data") short[] data) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.write(data);
    }

    public static void brickletOLED128x64Write(@Nullable ThingActions actions, short[] data)
            throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64Write(data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "New Window")
    public void brickletOLED128x64NewWindow(@ActionInput(name = "columnFrom") short columnFrom,
            @ActionInput(name = "columnTo") short columnTo, @ActionInput(name = "rowFrom") short rowFrom,
            @ActionInput(name = "rowTo") short rowTo) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.newWindow(columnFrom, columnTo, rowFrom, rowTo);
    }

    public static void brickletOLED128x64NewWindow(@Nullable ThingActions actions, short columnFrom, short columnTo,
            short rowFrom, short rowTo) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64NewWindow(columnFrom, columnTo, rowFrom, rowTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void brickletOLED128x64ClearDisplay() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.clearDisplay();
    }

    public static void brickletOLED128x64ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64ClearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void brickletOLED128x64WriteLine(@ActionInput(name = "line") short line,
            @ActionInput(name = "position") short position, @ActionInput(name = "text") String text)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.writeLine(line, position, text);
    }

    public static void brickletOLED128x64WriteLine(@Nullable ThingActions actions, short line, short position,
            String text) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            ((BrickletOLED128x64Actions) actions).brickletOLED128x64WriteLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type = "short") @ActionOutput(name = "invert", type = "boolean") Map<String, Object> brickletOLED128x64GetDisplayConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletOLED128x64.DisplayConfiguration value = dev.getDisplayConfiguration();

        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64GetDisplayConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            return ((BrickletOLED128x64Actions) actions).brickletOLED128x64GetDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletOLED128x64GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletOLED128x64Wrapper dev = ((@Nullable BrickletOLED128x64Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletOLED128x64.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64Actions) {
            return ((BrickletOLED128x64Actions) actions).brickletOLED128x64GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64Actions class.");
        }
    }
}
