/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRGBLEDV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the RGB LED Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set RGB Value")
    public void brickletRGBLEDV2SetRGBValue(@ActionInput(name = "r") int r, @ActionInput(name = "g") int g,
            @ActionInput(name = "b") int b) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setRGBValue(r, g, b);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletRGBLEDV2Color")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletRGBLEDV2Color"), RefreshType.REFRESH);
        }
    }

    public static void brickletRGBLEDV2SetRGBValue(@Nullable ThingActions actions, int r, int g, int b)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2SetRGBValue(r, g, b);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get RGB Value")
    public @ActionOutput(name = "r", type = "int") @ActionOutput(name = "g", type = "int") @ActionOutput(name = "b", type = "int") Map<String, Object> brickletRGBLEDV2GetRGBValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDV2.RGBValue value = dev.getRGBValue();

        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetRGBValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetRGBValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletRGBLEDV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletRGBLEDV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletRGBLEDV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletRGBLEDV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRGBLEDV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDV2Wrapper dev = ((@Nullable BrickletRGBLEDV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }
}
