/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRotaryEncoderV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Rotary Encoder Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRotaryEncoderV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Count")
    public @ActionOutput(name = "count", type = "int") Map<String, Object> brickletRotaryEncoderV2GetCount(
            @ActionInput(name = "reset") boolean reset) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCount(reset);

        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetCount(@Nullable ThingActions actions, boolean reset)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetCount(reset);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type = "boolean") Map<String, Object> brickletRotaryEncoderV2IsPressed()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isPressed();

        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2IsPressed(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2IsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletRotaryEncoderV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletRotaryEncoderV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletRotaryEncoderV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletRotaryEncoderV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRotaryEncoderV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRotaryEncoderV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderV2Wrapper dev = ((@Nullable BrickletRotaryEncoderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRotaryEncoderV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderV2Actions) {
            return ((BrickletRotaryEncoderV2Actions) actions).brickletRotaryEncoderV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderV2Actions class.");
        }
    }
}
