/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletRotaryEncoder;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Rotary Encoder Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRotaryEncoderWrapper extends BrickletRotaryEncoder implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRotaryEncoder.DEVICE_DISPLAY_NAME,
            "brickletrotaryencoder", BrickletRotaryEncoder.DEVICE_IDENTIFIER, BrickletRotaryEncoderWrapper.class,
            BrickletRotaryEncoderActions.class, "2.0.0", false);

    public BrickletRotaryEncoderWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletRotaryEncoderWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRotaryEncoderConfig cfg = (BrickletRotaryEncoderConfig) config.as(BrickletRotaryEncoderConfig.class);
        this.addCountListener(this.reg((count) -> {
            if (true) {
                updateStateFn.accept("BrickletRotaryEncoderCount",
                        transformBrickletRotaryEncoderCountCallback0(count, cfg));
            }
        }, this::removeCountListener));
        this.addPressedListener(this.reg(() -> {
            if (true) {
                triggerChannelFn.accept("BrickletRotaryEncoderPressed",
                        transformBrickletRotaryEncoderPressedCallback0(cfg));
            }
        }, this::removePressedListener));
        this.addReleasedListener(this.reg(() -> {
            if (true) {
                triggerChannelFn.accept("BrickletRotaryEncoderPressed",
                        transformBrickletRotaryEncoderPressedCallback1(cfg));
            }
        }, this::removeReleasedListener));

        {
            BrickletRotaryEncoderCountConfig channelCfg = getChannelConfigFn.apply("BrickletRotaryEncoderCount")
                    .as(BrickletRotaryEncoderCountConfig.class);
            this.setCountCallbackPeriod(channelCfg.updateInterval);
            this.setCountCallbackThreshold('x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setCountCallbackPeriod(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletRotaryEncoderCount");
        result.add("BrickletRotaryEncoderResetCounter");
        result.add("BrickletRotaryEncoderPressed");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletRotaryEncoderCount":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletRotaryEncoderCount"), "Count",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRotaryEncoderCount"))
                        .withDescription(
                                "The current count of the encoder. The encoder has 24 steps per rotation. Turning the encoder to the left decrements the counter, so a negative count is possible.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletRotaryEncoderResetCounter":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletRotaryEncoderResetCounter"), "Reset Counter",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletRotaryEncoderResetCounter"))
                        .withDescription("Resets the counter to 0.").withCommandDescription(CommandDescriptionBuilder
                                .create().withCommandOption(new CommandOption("RESET", "Reset")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Rotary Encoder Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("360° rotary encoder with push-button")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletRotaryEncoderCount",
                                new ChannelTypeUID("tinkerforge", "BrickletRotaryEncoderCount")).withLabel("Count")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletRotaryEncoderResetCounter",
                                new ChannelTypeUID("tinkerforge", "BrickletRotaryEncoderResetCounter"))
                                        .withLabel("Reset Counter").build(),
                        new ChannelDefinitionBuilder("BrickletRotaryEncoderPressed",
                                new ChannelTypeUID("system", "rawbutton")).withLabel("Pressed")
                                        .withDescription("Triggers if the button is pressed or released").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletrotaryencoder":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletRotaryEncoderCount":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletRotaryEncoderResetCounter":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletRotaryEncoderConfig cfg = (BrickletRotaryEncoderConfig) config.as(BrickletRotaryEncoderConfig.class);
        switch (channel) {
            case "BrickletRotaryEncoderCount": {
                int value = this.getCount(false);
                if (true)
                    updateStateFn.accept(channel, transformBrickletRotaryEncoderCountGetter0(value, cfg));
            }
                return;
            case "BrickletRotaryEncoderResetCounter":
                return;
            case "BrickletRotaryEncoderPressed": {
                boolean value = this.isPressed();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletRotaryEncoderPressedGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletRotaryEncoderResetCounter":
                if (command instanceof StringType) {
                    this.getCount(true);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletRotaryEncoderCount", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletRotaryEncoderCountCallback0(int count,
            BrickletRotaryEncoderConfig cfg) {
        return new DecimalType(count);
    }

    private String transformBrickletRotaryEncoderPressedCallback0(BrickletRotaryEncoderConfig cfg) {
        return CommonTriggerEvents.PRESSED;
    }

    private String transformBrickletRotaryEncoderPressedCallback1(BrickletRotaryEncoderConfig cfg) {
        return CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletRotaryEncoderCountGetter0(int value,
            BrickletRotaryEncoderConfig cfg) {
        return new DecimalType(value);
    }

    private String transformBrickletRotaryEncoderPressedGetter0(boolean value, BrickletRotaryEncoderConfig cfg) {
        return value ? CommonTriggerEvents.PRESSED : CommonTriggerEvents.RELEASED;
    }
}
