/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletSegmentDisplay4x7;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Segment Display 4x7 Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSegmentDisplay4x7Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Segments")
    public void brickletSegmentDisplay4x7SetSegments(@ActionInput(name = "segments") short[] segments,
            @ActionInput(name = "brightness") short brightness, @ActionInput(name = "colon") boolean colon)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSegmentDisplay4x7Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSegments(segments, brightness, colon);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Segments")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Segments"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Colon")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Colon"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Brightness")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7Brightness"), RefreshType.REFRESH);
        }
    }

    public static void brickletSegmentDisplay4x7SetSegments(@Nullable ThingActions actions, short[] segments,
            short brightness, boolean colon) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7SetSegments(segments, brightness,
                    colon);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Segments")
    public @ActionOutput(name = "segments", type = "short[]") @ActionOutput(name = "brightness", type = "short") @ActionOutput(name = "colon", type = "boolean") Map<String, Object> brickletSegmentDisplay4x7GetSegments()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSegmentDisplay4x7.Segments value = dev.getSegments();

        result.put("segments", value.segments);
        result.put("brightness", value.brightness);
        result.put("colon", value.colon);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetSegments(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetSegments();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Start Counter")
    public void brickletSegmentDisplay4x7StartCounter(@ActionInput(name = "valueFrom") short valueFrom,
            @ActionInput(name = "valueTo") short valueTo, @ActionInput(name = "increment") short increment,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletSegmentDisplay4x7Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.startCounter(valueFrom, valueTo, increment, length);
    }

    public static void brickletSegmentDisplay4x7StartCounter(@Nullable ThingActions actions, short valueFrom,
            short valueTo, short increment, long length) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7StartCounter(valueFrom, valueTo,
                    increment, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Value")
    public @ActionOutput(name = "value", type = "int") Map<String, Object> brickletSegmentDisplay4x7GetCounterValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCounterValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetCounterValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetCounterValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletSegmentDisplay4x7GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSegmentDisplay4x7.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }
}
