/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletThermalImaging;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Thermal Imaging Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermalImagingActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get High Contrast Image")
    public @ActionOutput(name = "image", type = "int[]") Map<String, Object> brickletThermalImagingGetHighContrastImage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int[] value = dev.getHighContrastImage();

        result.put("image", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetHighContrastImage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetHighContrastImage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Temperature Image")
    public @ActionOutput(name = "image", type = "int[]") Map<String, Object> brickletThermalImagingGetTemperatureImage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int[] value = dev.getTemperatureImage();

        result.put("image", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetTemperatureImage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetTemperatureImage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Statistics")
    public @ActionOutput(name = "spotmeterStatistics", type = "int[]") @ActionOutput(name = "temperatures", type = "int[]") @ActionOutput(name = "resolution", type = "int") @ActionOutput(name = "ffcStatus", type = "int") @ActionOutput(name = "temperatureWarning", type = "boolean[]") Map<String, Object> brickletThermalImagingGetStatistics()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermalImaging.Statistics value = dev.getStatistics();

        result.put("spotmeterStatistics", value.spotmeterStatistics);
        result.put("temperatures", value.temperatures);
        result.put("resolution", value.resolution);
        result.put("ffcStatus", value.ffcStatus);
        result.put("temperatureWarning", value.temperatureWarning);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetStatistics(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetStatistics();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Resolution")
    public @ActionOutput(name = "resolution", type = "int") Map<String, Object> brickletThermalImagingGetResolution()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getResolution();

        result.put("resolution", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetResolution(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetResolution();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Spotmeter Config")
    public @ActionOutput(name = "regionOfInterest", type = "int[]") Map<String, Object> brickletThermalImagingGetSpotmeterConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int[] value = dev.getSpotmeterConfig();

        result.put("regionOfInterest", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetSpotmeterConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetSpotmeterConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get High Contrast Config")
    public @ActionOutput(name = "regionOfInterest", type = "int[]") @ActionOutput(name = "dampeningFactor", type = "int") @ActionOutput(name = "clipLimit", type = "int[]") @ActionOutput(name = "emptyCounts", type = "int") Map<String, Object> brickletThermalImagingGetHighContrastConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermalImaging.HighContrastConfig value = dev.getHighContrastConfig();

        result.put("regionOfInterest", value.regionOfInterest);
        result.put("dampeningFactor", value.dampeningFactor);
        result.put("clipLimit", value.clipLimit);
        result.put("emptyCounts", value.emptyCounts);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetHighContrastConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetHighContrastConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletThermalImagingReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletThermalImagingGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletThermalImagingGetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletThermalImagingGetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermalImaging.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletThermalImagingGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermalImagingWrapper dev = ((@Nullable BrickletThermalImagingWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermalImaging.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermalImagingGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).brickletThermalImagingGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }
}
