/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletThermocoupleV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Thermocouple Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletThermocoupleV2GetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type = "int") @ActionOutput(name = "thermocoupleType", type = "int") @ActionOutput(name = "filter", type = "int") Map<String, Object> brickletThermocoupleV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocoupleV2.Configuration value = dev.getConfiguration();

        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type = "boolean") @ActionOutput(name = "openCircuit", type = "boolean") Map<String, Object> brickletThermocoupleV2GetErrorState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocoupleV2.ErrorState value = dev.getErrorState();

        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetErrorState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetErrorState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletThermocoupleV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletThermocoupleV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocoupleV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletThermocoupleV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocoupleV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }
}
