/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletVoltageCurrentV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Voltage/Current Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletVoltageCurrentV2GetVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type = "int") Map<String, Object> brickletVoltageCurrentV2GetCurrent()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCurrent();

        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCurrent(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type = "int") Map<String, Object> brickletVoltageCurrentV2GetPower()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getPower();

        result.put("power", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetPower(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type = "int") @ActionOutput(name = "voltageConversionTime", type = "int") @ActionOutput(name = "currentConversionTime", type = "int") Map<String, Object> brickletVoltageCurrentV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Configuration value = dev.getConfiguration();

        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "voltageMultiplier", type = "int") @ActionOutput(name = "voltageDivisor", type = "int") @ActionOutput(name = "currentMultiplier", type = "int") @ActionOutput(name = "currentDivisor", type = "int") Map<String, Object> brickletVoltageCurrentV2GetCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Calibration value = dev.getCalibration();

        result.put("voltageMultiplier", value.voltageMultiplier);
        result.put("voltageDivisor", value.voltageDivisor);
        result.put("currentMultiplier", value.currentMultiplier);
        result.put("currentDivisor", value.currentDivisor);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletVoltageCurrentV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletVoltageCurrentV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletVoltageCurrentV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }
}
