/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-05-19.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletXMC1400Breakout;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the XMC1400 Breakout Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletXMC1400BreakoutWrapper extends BrickletXMC1400Breakout implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletXMC1400Breakout.DEVICE_DISPLAY_NAME,
            "brickletxmc1400breakout", BrickletXMC1400Breakout.DEVICE_IDENTIFIER, BrickletXMC1400BreakoutWrapper.class,
            BrickletXMC1400BreakoutActions.class, "2.0.0", true);

    public BrickletXMC1400BreakoutWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletXMC1400BreakoutWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletXMC1400BreakoutConfig cfg = (BrickletXMC1400BreakoutConfig) config
                .as(BrickletXMC1400BreakoutConfig.class);
        this.addCountListener(this.reg((count) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutCount",
                        transformBrickletXMC1400BreakoutCountCallback0(count, cfg));
            }
        }, this::removeCountListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue0",
                        transformBrickletXMC1400BreakoutADCValue0Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue1",
                        transformBrickletXMC1400BreakoutADCValue1Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue2",
                        transformBrickletXMC1400BreakoutADCValue2Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue3",
                        transformBrickletXMC1400BreakoutADCValue3Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue4",
                        transformBrickletXMC1400BreakoutADCValue4Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue5",
                        transformBrickletXMC1400BreakoutADCValue5Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue6",
                        transformBrickletXMC1400BreakoutADCValue6Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.addADCValuesListener(this.reg((values) -> {
            if (true) {
                updateStateFn.accept("BrickletXMC1400BreakoutADCValue7",
                        transformBrickletXMC1400BreakoutADCValue7Callback0(values, cfg));
            }
        }, this::removeADCValuesListener));
        this.setADCValuesCallbackConfiguration(cfg.adcValuesUpdateInterval, true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletXMC1400BreakoutCountConfig channelCfg = getChannelConfigFn.apply("BrickletXMC1400BreakoutCount")
                    .as(BrickletXMC1400BreakoutCountConfig.class);
            this.setCountCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setCountCallbackConfiguration(0, true, 'x', 0, 0);
        this.setADCValuesCallbackConfiguration(0, true);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletXMC1400BreakoutCount");
        result.add("BrickletXMC1400BreakoutADCValue0");
        result.add("BrickletXMC1400BreakoutADCValue1");
        result.add("BrickletXMC1400BreakoutADCValue2");
        result.add("BrickletXMC1400BreakoutADCValue3");
        result.add("BrickletXMC1400BreakoutADCValue4");
        result.add("BrickletXMC1400BreakoutADCValue5");
        result.add("BrickletXMC1400BreakoutADCValue6");
        result.add("BrickletXMC1400BreakoutADCValue7");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletXMC1400BreakoutCount":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutCount"), "Count",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletXMC1400BreakoutCount"))
                        .withDescription("The value of the example count (see example.c)")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletXMC1400BreakoutADCValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"), "ADC Value",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletXMC1400BreakoutADCValue"))
                        .withDescription("The value for one of the 8 ADC channels of the adc driver example.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge XMC1400 Breakout Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Breakout for Infineon XMC1400 microcontroller")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutCount",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutCount")).withLabel("Count")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue0",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 0").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue1",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 1").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue2",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 2").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue3",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 3").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue4",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 4").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue5",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 5").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue6",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 6").build(),
                        new ChannelDefinitionBuilder("BrickletXMC1400BreakoutADCValue7",
                                new ChannelTypeUID("tinkerforge", "BrickletXMC1400BreakoutADCValue"))
                                        .withLabel("ADC Value 7").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletxmc1400breakout":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("adcValuesUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for ADC values in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("ADC Values Update Interval")
                                .withUnit("ms").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletXMC1400BreakoutCount":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletXMC1400BreakoutADCValue":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletXMC1400BreakoutConfig cfg = (BrickletXMC1400BreakoutConfig) config
                .as(BrickletXMC1400BreakoutConfig.class);
        switch (channel) {
            case "BrickletXMC1400BreakoutCount": {
                long value = this.getCount();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutCountGetter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue0": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue0Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue1": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue1Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue2": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue2Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue3": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue3Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue4": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue4Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue5": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue5Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue6": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue6Getter0(value, cfg));
            }
                return;
            case "BrickletXMC1400BreakoutADCValue7": {
                int[] value = this.getADCValues();
                if (true)
                    updateStateFn.accept(channel, transformBrickletXMC1400BreakoutADCValue7Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutCountCallback0(long count,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(count);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue0Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[0]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue1Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[1]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue2Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[2]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue3Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[3]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue4Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[4]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue5Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[5]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue6Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[6]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue7Callback0(int[] values,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(values[7]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutCountGetter0(long value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue0Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[0]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue1Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[1]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue2Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[2]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue3Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[3]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue4Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[4]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue5Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[5]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue6Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[6]);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletXMC1400BreakoutADCValue7Getter0(int[] value,
            BrickletXMC1400BreakoutConfig cfg) {
        return new DecimalType(value[7]);
    }
}
